@@ -1,6 +1,6 @@
-/* $Id: ifp.c,v 1.37 2004/11/05 18:57:14 yamajun Exp $ */
+/* $Id: ifp.c,v 1.38 2004/12/18 03:24:26 yamajun Exp $ */
 
-static char const rcsid[] = "$Id: ifp.c,v 1.37 2004/11/05 18:57:14 yamajun Exp $";
+static char const rcsid[] = "$Id: ifp.c,v 1.38 2004/12/18 03:24:26 yamajun Exp $";
 
 #include <stdio.h>
 #include <string.h>
@@ -51,7 +51,7 @@ int main(int argc, char **argv)
 	    
     usb_init();
 
-    if(usb_find_busses() <= 0) {
+    if(usb_find_busses() < 0) {
 	fprintf(stderr, "Could not find any USB busses.\n");
 	return 1;
     }
