@@ -1,6 +1,6 @@
-/* $Id: ifp_routines.c,v 1.50 2004/11/05 16:44:22 yamajun Exp $ */
+/* $Id: ifp_routines.c,v 1.55 2005/05/08 03:54:32 yamajun Exp $ */
 
-static const char rcsid[] = "$Id: ifp_routines.c,v 1.50 2004/11/05 16:44:22 yamajun Exp $";
+static const char rcsid[] = "$Id: ifp_routines.c,v 1.55 2005/05/08 03:54:32 yamajun Exp $";
 
 #include <stdio.h>
 #include <string.h>
@@ -81,9 +81,10 @@ void slash2backslash(char *dest, const char *src, size_t count);
 int ifp_ctl_datasize(unsigned char *control);
 char *dump_control_regs(const char *control, int length);
 int safe_suffix(const char *name);
-// XXX Thats two functions not used.
-int ifp_file_exists(usb_dev_handle *dh, const char *name,int type);
-int get_filepath(char *dest, const char *path);
+int firmware_check_ifp(const char *firm_name);
+int firmware_check_n10(const char *firm_name);
+int ifp_file_exists(usb_dev_handle *dh, const char *name);
+int get_dirname(char *dest, const char *path);
 
 
 typedef struct dirlists {
@@ -105,7 +106,9 @@ int ifp_put(usb_dev_handle *dh, int argc, char *argv[]) {
     if (argc < 2) {
 	printf("usage: put localfile\n");
 	printf("       put localdir\n");
+	return -1;
     }
+
     nargv[0] = argv[0];
     nargv[1] = argv[1];
 
@@ -116,7 +119,7 @@ int ifp_put(usb_dev_handle *dh, int argc, char *argv[]) {
 	    nargv[2] = get_basename(argv[1]);
 	}
     } else {
-	perror("unable to upload file");
+	perror("Unable to upload file");
 	return -1;
     }
 
@@ -151,10 +154,11 @@ int ifp_upload(usb_dev_handle *dh, int argc, char *argv[]) {
 
 	    if (ifp_isdir(dh, ifp_path)) {
 		// file to dir
-		strncpy(ifp_path, argv[2], IFP_MAXPATHLEN - 2);
-		strncat(ifp_path, "/", 1);
-		strncat(ifp_path, get_basename(argv[1]),
-			IFP_MAXPATHLEN - strlen(ifp_path) - 1);
+		snprintf(ifp_path, IFP_MAXPATHLEN, "%s/%s", argv[2], get_basename(argv[1]));
+		//strncpy(ifp_path, argv[2], IFP_MAXPATHLEN - 2);
+		//strncat(ifp_path, "/", 1);
+		//strncat(ifp_path, get_basename(argv[1]),
+		//	IFP_MAXPATHLEN - strlen(ifp_path) - 1);
 		normalize_path(ifp_path, IFP_MAXPATHLEN);
 
 		retval = ifp_upload_file(dh, IS_IFP, ifp_path, fp);
@@ -201,16 +205,21 @@ int ifp_upload_file(usb_dev_handle *dh, int flag, const char *name, FILE *f) {
     fstat(fileno(f),&stat);
     if (stat.st_size > ifp_get_free(dh)) {
 	// not enough space
+	fprintf(stderr, "%s: Not enough space.  Cannot upload.\n", name);
 	return -2;
     } else if (strlen(name) >= IFP_MAXPATHLEN) {
 	// Pathname too long
+	fprintf(stderr, "%s: Pathname too long.  Cannot upload.\n", name);
 	return -3;
     }
 
     rewind(f);
     ifp_send_filename(dh,name);
     usb_control_msg(dh,IFP_REQ_TYPE,IFP_FILE_UPLOAD,filesize,0,ctl,sizeof(ctl),TOUT);
-    if (!ifp_ok(ctl)) return -1;
+    if (!ifp_ok(ctl)) {
+	fprintf(stderr, "%s: Cannot upload.\n", name);
+	return -1;
+    }
 
     while (!feof(f)) {
 	i = fread(buf,1,sizeof(buf),f);
@@ -247,16 +256,12 @@ int ifp_get(usb_dev_handle *dh, int argc, char *argv[]) {
     if (argc < 2) {
 	printf("usage: get ifpfile\n");
 	printf("       get ifpdir\n");
+	return -1;
     }
 
     nargv[0] = argv[0];
     nargv[1] = argv[1];
-
-    if (ifp_isdir(dh, argv[1])) {
-	nargv[2] = ".";
-    } else {
-	nargv[2] = argv[1];
-    }
+    nargv[2] = ".";		// set destination
 
     return ifp_download(dh, 3, nargv);
 }
@@ -275,6 +280,11 @@ int ifp_download(usb_dev_handle *dh, int argc, char *argv[]) {
     strncpy(ifp_path, argv[1], IFP_MAXPATHLEN - 1);
     normalize_path(ifp_path, IFP_MAXPATHLEN);
 
+    if (!ifp_file_exists(dh, ifp_path)) {
+	fprintf(stderr, "ifp: No such file or directory in iFP: %s\n", ifp_path);
+	return -1;
+    }
+
     switch (filestat(argv[2])) {
     case 0: // "argv[2]" does not exist. create new file.
 	// file to file
@@ -322,16 +332,18 @@ int ifp_download(usb_dev_handle *dh, int argc, char *argv[]) {
     case 2: // directory
 	if (ifp_isdir(dh, ifp_path)) {
 	    // dir to dir
-	    strncpy(path_buf, argv[2], PATH_MAX - 2);
-	    strncat(path_buf, "/", 1);
-	    strncat(path_buf, get_basename(ifp_path), PATH_MAX - strlen(path_buf) - 1);
+	    snprintf(path_buf, PATH_MAX, "%s/%s", argv[2], get_basename(ifp_path));
+	    //strncpy(path_buf, argv[2], PATH_MAX - 2);
+	    //strncat(path_buf, "/", 1);
+	    //strncat(path_buf, get_basename(ifp_path), PATH_MAX - strlen(path_buf) - 1);
 	    retval = ifp_download_dir(dh, ifp_path, path_buf);
 
 	} else {
 	    // file to dir
-	    strncpy(path_buf, argv[2], PATH_MAX - 2);
-	    strncat(path_buf, "/", 1);
-	    strncat(path_buf, get_basename(ifp_path), PATH_MAX - strlen(path_buf) - 1);
+	    snprintf(path_buf, PATH_MAX, "%s/%s", argv[2], get_basename(ifp_path));
+	    //strncpy(path_buf, argv[2], PATH_MAX - 2);
+	    //strncat(path_buf, "/", 1);
+	    //strncat(path_buf, get_basename(ifp_path), PATH_MAX - strlen(path_buf) - 1);
 	    if (!safe_suffix(path_buf)) {
 		break;
 	    }
@@ -705,8 +717,7 @@ int ifp_firmware_update(usb_dev_handle *dh, int argc, char *argv[]) {
     struct usb_device *dev = usb_device(dh);
     FILE *fp;
     int retval;
-    unsigned char buf[FIRMWARE_HEADER_SIZE];
-    unsigned char checkdata[FIRMWARE_HEADER_SIZE] = {0x39, 0xb0, 0x5d, 0xed};
+    char buf[256];
     unsigned char ctl[4];
     char *basename;
 
@@ -725,32 +736,29 @@ int ifp_firmware_update(usb_dev_handle *dh, int argc, char *argv[]) {
 	}
     }
 
-    // send firmware file.
-    basename = get_basename(argv[1]);
-
-    if (strncmp(basename, "IFP-", 4) != 0) {
-	fprintf(stderr, "Firmware filename must be set \"IFP-?XXT.HEX\" or \"IFP-1XXTC.HEX\".\n");
+    if (ifp_type_string(dh, buf) < 0) {
 	return -1;
     }
+    if (strncmp(buf, "IFP-", 4) == 0) {
+	if (firmware_check_ifp(argv[1]) < 0) {
+	    fprintf(stderr, "Failed firmware consistency check.\n");
+	    return -1;
+	}
+    } else {
+	if (firmware_check_n10(argv[1]) < 0) {
+	    fprintf(stderr, "Failed firmware consistency check.\n");
+	    return -1;
+	}
+    }
+
+    // send firmware file.
     if ( (fp = fopen(argv[1], "r")) == NULL) {
 	perror(argv[1]);
 	return -1;
     }
 
-    if ( fread(buf, sizeof(unsigned char), sizeof(checkdata), fp)
-	    < FIRMWARE_HEADER_SIZE) {
-	fprintf(stderr, "ifp firmupdate: invalid firmware file(too short!).\n");
-	fclose(fp);
-	return -1;
-    }
-    rewind(fp);	    // DO NOT REMOVE IT!
-
-    if (strncmp(buf, checkdata, FIRMWARE_HEADER_SIZE) != 0) {
-	fprintf(stderr, "ifp firmupdate: Invalid format firmware file.\n");
-	fclose(fp);
-	return -1;
-    }
-
+    // Send firmware file to /FIRMWARE.HEX in iFP.
+    basename = get_basename(argv[1]);
     if ( (retval = ifp_upload_file(dh, IS_IFP, basename, fp)) < 0) {
 	fprintf(stderr, "Failed firmware upload.\n");
 	fclose(fp);
@@ -837,21 +845,27 @@ int ifp_set_tuner_preset(usb_dev_handle *dh, int argc, char *argv[]) {
     if (argc < 2) {
 	printf("usage: setpreset myarea.dat\n");
 	return -1;
-    } else {
-	if (ifp_firmware_version(dh) < 200) {
-	    fprintf(stderr, "Does not support this firmware(before v2.00).\n");
-	} else {
-	    printf("WARNING: Do you want to change FM preset? [y/N]: ");
-	    switch (getc(stdin)) {
-	    case 'Y':	/* yes */
-	    case 'y':
-		break;
-	    default:	/* no */
-		return -1;
-		break;
-	    }
+    } 
+
+    ifp_type_string(dh, buf);
+    // No FM station name display in iFP-1xx with old firmware(before 2.00).
+    if (strstr(buf, "IFP-1") != NULL) {
+	if ((ifp_firmware_version(dh) < 200)) {
+	    fprintf(stderr, "Does not support this command with old firmware(before v2.00).\n");
+	    return -1;
 	}
     }
+    
+    printf("WARNING: Do you want to change FM preset? [y/N]: ");
+    switch (getc(stdin)) {
+    case 'Y':	/* yes */
+    case 'y':
+	break;
+    default:	/* no */
+	return -1;
+	break;
+    }
+
     if ( (fp = fopen(argv[1], "r")) == NULL) {
 	perror(argv[1]);
 	return -1;
@@ -915,9 +929,10 @@ int ifp_upload_dir(usb_dev_handle *dh, const char *from, const char *name) {
 	toname = ftsentp->fts_path + basename_pos;
 
 	// "path" "file" -> "path/file"
-	strncpy(ifp_path, name, IFP_MAXPATHLEN - 2);
-	strncat(ifp_path, "/", 1);
-	strncat(ifp_path, toname, IFP_MAXPATHLEN - strlen(ifp_path) - 1);
+	snprintf(ifp_path, IFP_MAXPATHLEN, "%s/%s", name, toname);
+	//strncpy(ifp_path, name, IFP_MAXPATHLEN - 2);
+	//strncat(ifp_path, "/", 1);
+	//strncat(ifp_path, toname, IFP_MAXPATHLEN - strlen(ifp_path) - 1);
 
 	switch (ftsentp->fts_info) {
 	case FTS_F:		// normal file
@@ -1000,13 +1015,15 @@ int ifp_download_dir(usb_dev_handle *dh, const char *fromdir, const char *todir)
 	    unicode2string(strbuf,inbuf);
 
 	    // "path" "file" -> "path/file"
-	    strncpy(ifp_path, fromdir, IFP_MAXPATHLEN - 2);
-	    strncat(ifp_path, "/", 1);
-	    strncat(ifp_path, strbuf, IFP_MAXPATHLEN - strlen(ifp_path) - 1);
+	    snprintf(ifp_path, IFP_MAXPATHLEN, "%s/%s", fromdir, strbuf);
+	    //strncpy(ifp_path, fromdir, IFP_MAXPATHLEN - 2);
+	    //strncat(ifp_path, "/", 1);
+	    //strncat(ifp_path, strbuf, IFP_MAXPATHLEN - strlen(ifp_path) - 1);
 
-	    strncpy(to_path, todir, IFP_MAXPATHLEN - 2);
-	    strncat(to_path, "/", 1);
-	    strncat(to_path, strbuf, IFP_MAXPATHLEN - strlen(to_path) - 1);
+	    snprintf(to_path, IFP_MAXPATHLEN, "%s/%s", todir, strbuf);
+	    //strncpy(to_path, todir, IFP_MAXPATHLEN - 2);
+	    //strncat(to_path, "/", 1);
+	    //strncat(to_path, strbuf, IFP_MAXPATHLEN - strlen(to_path) - 1);
 
 	    switch (filestat(to_path)) {
 	    case 0:	// "to_path" not found;
@@ -1046,9 +1063,10 @@ int ifp_download_dir(usb_dev_handle *dh, const char *fromdir, const char *todir)
 	    unicode2string(strbuf,inbuf);
 
 	    // "path" "file" -> "path/file"
-	    strncpy(ifp_path, fromdir, IFP_MAXPATHLEN - 2);
-	    strncat(ifp_path, "/", 1);
-	    strncat(ifp_path, get_basename(strbuf), IFP_MAXPATHLEN - strlen(ifp_path) - 1);
+	    snprintf(ifp_path, IFP_MAXPATHLEN, "%s/%s", fromdir, strbuf);
+	    //strncpy(ifp_path, fromdir, IFP_MAXPATHLEN - 2);
+	    //strncat(ifp_path, "/", 1);
+	    //strncat(ifp_path, get_basename(strbuf), IFP_MAXPATHLEN - strlen(ifp_path) - 1);
 	    retval = push_dir(ifp_path);
 
 	    break;
@@ -1066,9 +1084,10 @@ LS_END:
     usb_control_msg(dh, IFP_REQ_TYPE, IFP_LS_LEAVE, 0, 0, ctl, sizeof(ctl), TOUT);
     while (pop_dir(strbuf)) {
 	// "path" "file" -> "path/file"
-	strncpy(to_path, todir, IFP_MAXPATHLEN - 2);
-	strncat(to_path, "/", 1);
-	strncat(to_path, get_basename(strbuf), IFP_MAXPATHLEN - strlen(to_path) - 1);
+	snprintf(to_path, IFP_MAXPATHLEN, "%s/%s", todir, strbuf);
+	//strncpy(to_path, todir, IFP_MAXPATHLEN - 2);
+	//strncat(to_path, "/", 1);
+	//strncat(to_path, get_basename(strbuf), IFP_MAXPATHLEN - strlen(to_path) - 1);
 
 	retval = ifp_download_dir(dh, strbuf, to_path);
 	if (retval == -1) {
@@ -1123,9 +1142,10 @@ int ifp_delete_recursive(usb_dev_handle *dh, char *name) {
 	    unicode2string(strbuf,inbuf);
 
 	    // "path" "file" -> "path/file"
-	    strncpy(ifp_path, name, IFP_MAXPATHLEN - 2);
-	    strncat(ifp_path, "/", 1);
-	    strncat(ifp_path, strbuf, IFP_MAXPATHLEN - strlen(ifp_path) - 1);
+	    snprintf(ifp_path, IFP_MAXPATHLEN, "%s/%s", name, strbuf);
+	    //strncpy(ifp_path, name, IFP_MAXPATHLEN - 2);
+	    //strncat(ifp_path, "/", 1);
+	    //strncat(ifp_path, strbuf, IFP_MAXPATHLEN - strlen(ifp_path) - 1);
 	    retval = ifp_delete_file(dh, ifp_path);
 
 	    break;
@@ -1135,9 +1155,10 @@ int ifp_delete_recursive(usb_dev_handle *dh, char *name) {
 	    unicode2string(strbuf,inbuf);
 
 	    // "path" "file" -> "path/file"
-	    strncpy(ifp_path, name, IFP_MAXPATHLEN - 2);
-	    strncat(ifp_path, "/", 1);
-	    strncat(ifp_path, strbuf, IFP_MAXPATHLEN - strlen(ifp_path) - 1);
+	    snprintf(ifp_path, IFP_MAXPATHLEN, "%s/%s", name, strbuf);
+	    //strncpy(ifp_path, name, IFP_MAXPATHLEN - 2);
+	    //strncat(ifp_path, "/", 1);
+	    //strncat(ifp_path, strbuf, IFP_MAXPATHLEN - strlen(ifp_path) - 1);
 	    retval = push_dir(ifp_path);
 
 	    break;
@@ -1251,6 +1272,13 @@ int ifp_send_data(usb_dev_handle *dh, char *data, size_t size) {
     return usb_bulk_write(dh,IFP_BULK_TO,data,size,TOUT);
 }
 
+/**
+ * File exists checker(only for file in PC)
+ * @param   name    Filename in PC
+ * @retval  0	File is not exist.
+ * @retval  1	File is a regular file.
+ * @retval  2	File is a directory.
+ */
 int filestat(const char *name) {
     struct stat status;
 
@@ -1349,6 +1377,23 @@ char *get_basename(const char *path) {
     return start;
 }
 
+/*
+ * Copy path's dirname to dest
+ * and return number of copied charactors
+ */
+int get_dirname(char *dest, const char *path) {
+    int i;
+    int lastslash;
+    for (i = 0; *path != '\0'; i++) {
+	dest[i] = *path++;
+	if ('/' == dest[i]) {
+	    lastslash = i;
+	}
+    }
+    dest[lastslash] = '\0';
+    return lastslash;
+}
+
 char *ifp_strcasestr(const char *base, const char *target) {
     size_t len;
 
@@ -1444,15 +1489,23 @@ char *dump_control_regs(const char *control, int length) {
 }
 
 /**
- * TYPE 1 == File
- * TYPE 2 == Directory
- * RETURNS
- * 1 if exists and is file
- * 2 if exists and is dir
- * SAME AS DEVICE RETURNS FROM IFP_LS_NEXT
+ * Check file exists in iFP and filetype.
+ *
+ * usage:
+ * // !1 and !2 is same value(zero) in C
+ * path = "/path/to/file/mp3";
+ * if (!ifp_file_exists(dh, "path") {
+ *     fprintf(stderr, "%s is not exist\n", path);
+ * }
+ *
+ * @param[in]   dh	USB device handle
+ * @param[in]   name	file path in iFP
+ * @retval	0	"name" is not exists or error occured
+ * @retval	1	"name" is exists and it's a file.
+ * @retval	2	"name" is exists and it's a directory.
  */
-int ifp_file_exists(usb_dev_handle *dh, const char *name, int type) {
-    int retval = 0;
+int ifp_file_exists(usb_dev_handle *dh, const char *name) {
+    int retval = 0;		// "name" isn't exists yet
     int ret = 0;
     unsigned char ctl[4];
     unsigned char lsbuf[IFP_LSBUFLEN];
@@ -1460,23 +1513,25 @@ int ifp_file_exists(usb_dev_handle *dh, const char *name, int type) {
     unsigned char path[IFP_MAXPATHLEN];
     unsigned char pathbuf[IFP_MAXPATHLEN];
 
+    // path/to -> /path/to
     if (name[0] == '/')
 	strcat(pathbuf,name);
     else {
 	pathbuf[0] = '/';
 	strcat(pathbuf,name);
-    }	
-    get_filepath(path,pathbuf);
+    }
+    // /path/to -> /path
+    get_dirname(path, pathbuf);
 	    
-    if (ifp_send_filename(dh,path) < 0) {
-	fprintf(stderr, "ERROR: Could not send filename: %s.\n", path);
-	return -1;
+    if (ifp_send_filename(dh, path) < 0) {
+	fprintf(stderr, "ERROR: Could not send dirname: %s.\n", path);
+	return 0;
     }
     
     usb_control_msg(dh,IFP_REQ_TYPE,IFP_LS,0,0,ctl,sizeof(ctl),TOUT);
     if (!ifp_ok(ctl)) {
 	fprintf(stderr,"ERROR: LS command failed in ifp_file_exists. name = %s, path = %s, ctl = %s.\n", name,path,dump_control_regs(ctl,4));
-	return -1;
+	return 0;
     }
     
     while (1) {
@@ -1492,8 +1547,8 @@ int ifp_file_exists(usb_dev_handle *dh, const char *name, int type) {
 	    //if (type != 1) break;
 	    usb_bulk_read(dh,IFP_BULK_FROM,lsbuf,sizeof(lsbuf),TOUT);
 	    unicode2string(ascii_lsbuf,lsbuf);
-	    if (strcmp(ascii_lsbuf,get_basename(name)) == 0) {
-		retval = 1;
+	    if (strcmp(ascii_lsbuf, get_basename(name)) == 0) {
+		retval = 1;	// "name" is exists and it's a file
 		goto LS_END;
 	    }
 	    break;
@@ -1502,15 +1557,15 @@ int ifp_file_exists(usb_dev_handle *dh, const char *name, int type) {
 	    //if (type != 2) break;
 	    usb_bulk_read(dh,IFP_BULK_FROM,lsbuf,sizeof(lsbuf),TOUT);
 	    unicode2string(ascii_lsbuf,lsbuf);
-	    if (strcmp(ascii_lsbuf,get_basename(name)) == 0) {
-		retval = 2;
+	    if (strcmp(ascii_lsbuf, get_basename(name)) == 0) {
+		retval = 2;	// "name" is exists and it's a directory
 		goto LS_END;
 	    }
 	    break;
 
 	default:
 	    fprintf(stderr,"Uknown LS_NEXT return value = %d\n", ret);
-	    return -3;
+	    return 0;
 	}
     }
 
@@ -1518,26 +1573,12 @@ LS_END:
     usb_control_msg(dh,IFP_REQ_TYPE,IFP_LS_LEAVE,0,0,ctl,sizeof(ctl),TOUT);	
     if (!ifp_ok(ctl)) {
 	fprintf(stderr,"ERROR: LS LEAVE ****\n");
-	return -1;
+	return 0;
     }
 
     return retval;
 }
 
-int get_filepath(char *dest, const char *path) {
-    int i;
-    char *index = rindex(path,'/');
-							    
-    for (i = 0; (*path != '\0') && (path != index); i++) {
-	dest[i] = *path;
-	path++;
-    }
-    
-    dest[++i] = '\0';
-
-    return i;
-}
-
 int safe_suffix(const char *name) {
     if ( (ifp_strcasestr(name, ".mp3") || 
 	  ifp_strcasestr(name, ".wma") || 
@@ -1549,3 +1590,74 @@ int safe_suffix(const char *name) {
     return 1;	    // true
 }
 
+int firmware_check_ifp(const char *firm_name) {
+    FILE *fp;
+    unsigned char magic[FIRMWARE_HEADER_SIZE] = {0x39, 0xb0, 0x5d, 0xed};
+    unsigned char buf[FIRMWARE_HEADER_SIZE];
+    char *basename;
+
+    basename = get_basename(firm_name);
+
+    if (strncmp(basename, "IFP-", 4) != 0) {
+	fprintf(stderr, "Firmware filename must be set \"IFP-?XXT.HEX\" or \"IFP-1XXTC.HEX\".\n");
+	return -1;
+    }
+
+    if ( (fp = fopen(firm_name, "r")) == NULL) {
+	perror(firm_name);
+	return -1;
+    }
+
+    if ( fread(buf, sizeof(unsigned char), sizeof(magic), fp)
+	    < FIRMWARE_HEADER_SIZE) {
+	fprintf(stderr, "ifp firmupdate: invalid firmware file(too short!).\n");
+	fclose(fp);
+	return -1;
+    }
+    rewind(fp);	    // DO NOT REMOVE IT!
+
+    if (strncmp(buf, magic, FIRMWARE_HEADER_SIZE) != 0) {
+	fprintf(stderr, "ifp firmupdate: Invalid format firmware file.\n");
+	fclose(fp);
+	return -1;
+    }
+
+    fclose(fp);
+    return 0;
+}
+
+int firmware_check_n10(const char *firm_name) {
+    FILE *fp;
+    unsigned char magic[8] = {0x37, 0x13, 0x0d, 0xda, 0x32, 0xa2, 0xab, 0x23};
+    unsigned char buf[8];
+    char *basename;
+
+    basename = get_basename(firm_name);
+
+    if (strncmp(basename, "N10.HEX", 7) != 0) {
+	fprintf(stderr, "Firmware filename must be set \"N10.HEX\".\n");
+	return -1;
+    }
+
+    if ( (fp = fopen(firm_name, "r")) == NULL) {
+	perror(firm_name);
+	return -1;
+    }
+
+    if ( fread(buf, sizeof(unsigned char), sizeof(magic), fp) < 8) {
+	fprintf(stderr, "ifp firmupdate: invalid firmware file(too short!).\n");
+	fclose(fp);
+	return -1;
+    }
+    rewind(fp);	    // DO NOT REMOVE IT!
+
+    if (strncmp(buf, magic, 8) != 0) {
+	fprintf(stderr, "ifp firmupdate: Invalid format firmware file.\n");
+	fclose(fp);
+	return -1;
+    }
+
+    fclose(fp);
+    return 0;
+}
+
